
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Graphics;
import java.awt.image.BufferedImage;
import java.util.Random;

import javax.swing.JFrame;
import javax.swing.JPanel;

public class DiffusionParticle{
	JFrame frame;//メインウィンドウ
    JPanel panel;

	BufferedImage image;//バッファ領域（ダブルバッファリング用）

	Graphics fg;//描画領域用描画オブジェクト（グラフィックスコンテキスト）
	Graphics bg;//ダブルバッファリング用描画オブジェクト（グラフィックスコンテキスト）

	int screenW;//ウィンドウの幅
	int screenH;//ウィンドウの高さ

	final int N=256;//フィールドサイズ
	final int N2=N/2;
	final int FIELD_ROWS=N+2;//フィールド幅
	final int FIELD_COLS=N+2;//フィールド高さ
	final int CELL_SIZE=2;//正方形（セル）のサイズ

	//フィールド
	int[][] field=new int[FIELD_ROWS][FIELD_COLS];

	Random rand;
	int particleNum=5000;

	int[][] particle=new int[particleNum][2]; //x,y,state(1:fix, 0:float)
	int fixedNum=0;

	//画面の初期化処理
	public void initScreen() {
		//画面サイズの設定
		screenW=N*CELL_SIZE;
		screenH=N*CELL_SIZE;

		//フレームの設定
		frame=new JFrame();
		frame.setTitle("Reaction Diffusion System");
		frame.setBounds(200,200,screenW,screenH);
		frame.setResizable(false);//ウィンドウサイズを変更不可に設定
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		Container cont=frame.getContentPane();
		cont.setLayout(new FlowLayout(FlowLayout.CENTER,0,0));
		panel=new JPanel();
		panel.setPreferredSize(new Dimension(screenW,screenH));
		cont.add(panel);

		frame.pack();
		frame.setVisible(true);

		//描画領域のサイズ調整
        panel.setPreferredSize(new Dimension(screenW,screenH));
		frame.pack();

		//フレームに描画する準備
		image=(BufferedImage)panel.createImage(screenW, screenH);
		fg=panel.getGraphics();
		bg=image.getGraphics();

	}

	//フィールド配列のクリア
	void clearField() {
		for(int row=1;row<=N;row++) {
			for(int col=1;col<=N;col++) {
				field[row][col]=0;
			}
		}
	}

	//初期条件の設定
	void setInitialCondition(){

		//初期パーティクルの設定
		//rand.nextInt(N);
		rand=new Random();
		for(int i=1;i<particleNum;i++) {
			int x=(int)(N2+(5.0*rand.nextDouble())*Math.cos(2*Math.PI*i/N));
			int y=(int)(N2+(5.0*rand.nextDouble())*Math.sin(2*Math.PI*i/N));

			particle[i][0]=(x<0?0:(x>N?N:x));
			particle[i][1]=(y<0?0:(y>N?N:y));
		}

		particle[0][0]=N2;
		particle[0][1]=N2;

	}

	public void drawParticle() {
		for(int k=0;k<particleNum;k++) {
			int vRgb=0xffffffff;

			for(int i=0;i<CELL_SIZE;i++) {
				for(int j=0;j<CELL_SIZE;j++) {
					image.setRGB(particle[k][0]*CELL_SIZE+j, particle[k][1]*CELL_SIZE+i, 0);
				}
			}

			int x;
			int y;

			switch(rand.nextInt(4)) {
			case 0:
				x=particle[k][0]+1;
				y=particle[k][1];
				break;
			case 1:
				x=particle[k][0]-1;
				y=particle[k][1];
				break;
			case 2:
				x=particle[k][0];
				y=particle[k][1]+1;
				break;
			case 3:
				x=particle[k][0];
				y=particle[k][1]-1;
				break;
			default:
				x=particle[k][0];
				y=particle[k][1];
				break;
			}

			particle[k][0]=(x<0?0:(x>=N?N-1:x));//xの範囲を０からNまでに制限
			particle[k][1]=(y<0?0:(y>=N?N-1:y));//xの範囲を０からNまでに制限


			//Vを表示
			for(int i=0;i<CELL_SIZE;i++) {
				for(int j=0;j<CELL_SIZE;j++) {
					image.setRGB(particle[k][0]*CELL_SIZE+j, particle[k][1]*CELL_SIZE+i, vRgb);
				}
			}
		}

		//バッファ領域のイメージを描画領域へ転送
		fg.drawImage(image, 0, 0, panel);
	}

	//フィールドの描画
	public void drawField() {
		for(int row=1;row<=N;row++) {
			for(int col=1;col<=N;col++) {
				byte pix=(byte)(field[row][col]);
				//Vを表示
				int vRgb=((0xff<<24)+(pix<<16)+(pix<<8)+pix);
				for(int i=0;i<CELL_SIZE;i++) {
					for(int j=0;j<CELL_SIZE;j++) {
						image.setRGB((col-1)*CELL_SIZE+j, (row-1)*CELL_SIZE+i, vRgb);
					}
				}
			}
		}

		//バッファ領域のイメージを描画領域へ転送
		fg.drawImage(image, 0, 0, panel);

	}

	public DiffusionParticle(){

		//画面の初期化
		initScreen();
		//計算用フィールドの初期化
		clearField();
		//初期条件の設定
		setInitialCondition();
		//フィールドの描画
		drawField();
		//粒子の描画（初期配置）
		drawParticle();

		while(true) {
			//粒子の描画
			drawParticle();
		}
	}

	public static void main(String[] args){
		new DiffusionParticle();
	}
}
